package lda_gibbs;

import java.io.*;
import java.util.Random;

/**
 * Computes KL divergence matrix between topic distributions of two runs.
 * Outputs a Matlab-file.
 *
 * Two arguments for the application are two serialized LDAParameterEstimation-classes,
 * which are automatically produced when LDAParameterEstimation is run.
 *
 * @author akyrola@cs.cmu.edu
 */
public class ComputeKLMatrix {

    public static void main(String[] args) throws Exception {
        String run1File = args[0];
        String run2File = args[1];

        ObjectInputStream or1 = new ObjectInputStream(new FileInputStream(run1File));
        ObjectInputStream or2 = new ObjectInputStream(new FileInputStream(run2File));

        LDAParameterEstimation estimate1 = (LDAParameterEstimation) or1.readObject();
        LDAParameterEstimation estimate2 = (LDAParameterEstimation) or2.readObject();

        if (estimate1.getSampleCounters().length != estimate2.getSampleCounters().length) {
            throw new RuntimeException("Topic counts do not agree!!");
        }

        int K = estimate1.getSampleCounters().length;

        double[][]  matrix = new double[K][K];

        int W = estimate1.getWordIdMap().getNumOfWords();
        int W2 = estimate1.getWordIdMap().getNumOfWords();

        if (W != W2) throw new RuntimeException("Word counts do not match!!");

        for(int i=0;  i<K; i++) {
            System.out.println(i + "/" + K);
            for(int j=0; j<K; j++) {
                matrix[i][j] = estimate1.getSampleCounters()[i].symmetricKLDivergence(estimate2.getSampleCounters()[j], W, estimate1.beta);
            }
        }



        String outFile;
        if (args.length > 2) {
            outFile = args[2];
        } else {
            // Write benchmark system output
            String s = new File(run2File).getName();
            s = s.replace(".", "_");
            outFile = s;
        }
        BufferedWriter wr = new BufferedWriter(new FileWriter(outFile));

        wr.write("KLD = [");

        for(int i=0;  i<K; i++) {
            for(int j=0; j<K; j++) {
                wr.write(matrix[i][j] + "  ");
            }
            wr.write(";...\n");
        }


        Random r = new Random();
        // Compute "random" topic with uniform distribution.
        wr.write("\n randomKl = [");

        for(int j=0; j<K; j++) {

            int total = estimate1.getSampleCounters()[j].getNumOfWords();
            IndexedCounter unifDist = new IndexedCounter(total);
            for(int i=0; i<total; i++) unifDist.increment(i, 1);

            IndexedCounter randomDist = new IndexedCounter(total);
            for(int i=0; i<total; i++) randomDist.increment(i, r.nextInt(100));
            double klrandom = randomDist.symmetricKLDivergence(estimate1.getSampleCounters()[j], W, estimate1.beta);
            double klunif = unifDist.symmetricKLDivergence(estimate1.getSampleCounters()[j], W, estimate1.beta);

            System.out.println(j + " : " + klrandom + "," + klunif);
            wr.write(" " + klrandom + " " + klunif + ";");
        }

        wr.write("];\n");
        wr.close();

    }


}
