package lda_gibbs;


import java.util.Random;

/**
 * Takes a set of LDA documents and samples topics for words
 * in them. Actual sampling code is in LDADocument.
 * @see lda_gibbs.LDADocument
 * @author akyrola@cs.cmu.edu, 2010
 */
public class LDADocumentSampler {

   private double alpha;
   private double beta;
   private int numOfTopics;
   private int totalWords;


   // Note: these are shared between all threads!!!
   private IndexedCounter[] chainCounters;

   public LDADocumentSampler( double alpha,
                                       double beta,
                                       int numOfTopics,
                                       int totalWords,
                                       IndexedCounter[] chainCounters) {
        this.alpha = alpha;
        this.beta = beta;
        this.numOfTopics = numOfTopics;
        this.totalWords = totalWords;
        this.chainCounters = chainCounters;
    }

    /**
     * Samples topics for each word with id % modulo == vMod.
     * (Each parallel sampler samples for different set of words.
     * @param docSet set of document
     * @param vMod
     * @param modulo
     */
    public void sample( DocumentSet docSet, int vMod, int modulo) {
        Random rand = new Random(System.currentTimeMillis());

        long t = System.currentTimeMillis();
        for(LDADocument doc : docSet.getDocuments()) {
            synchronized (doc) {
                doc.resampleTopics(rand, vMod, modulo,
                    chainCounters, numOfTopics, alpha, beta, totalWords);
            }
        }


    }
}
