package lda_gibbs.datamgnt;


import java.io.*;

import lda_gibbs.datamgnt.WordIdMap;
import lda_gibbs.LDADocument;

/**
 * Converts to GibbsLDA++ software format
 * @author akyrola
 *         Date: Dec 16, 2009
 */
public class ConvertToGibbsLDAPlusPlus {

    private WordIdMap wordIdMap;
    private File directory;

    public ConvertToGibbsLDAPlusPlus(File directory) {
        this.directory = directory;
        wordIdMap = new WordIdMap(new File(directory, "index.lidx"));
    }

    public void convert(File outputFile) throws IOException {
        BufferedWriter writer = new BufferedWriter(new FileWriter(outputFile));
        File[] files = directory.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String name) {
                if (name.startsWith(".")) return false;
                if (name.endsWith(".ldat")) {
                    return true;
                }
                return false;
            }
        });
        writer.write(files.length + "\n");
        for(File f:directory.listFiles()) {
            if (f.getName().endsWith(".ldat")) {
                LDADocument ldadoc = new LDADocument(f);
                int[] docwords = ldadoc.getWords();
                if (docwords.length == 0) throw new RuntimeException("File " + f.getName() + " was empty!");
                for(int wordId:docwords) {
                    writer.write(wordIdMap.getWord(wordId));
                    writer.write(" ");
                }
                writer.write("\n");
            }
        }
        writer.close();
    }

    public static void main(String[] args) throws Exception {
        ConvertToGibbsLDAPlusPlus conv = new ConvertToGibbsLDAPlusPlus(new File(args[0]));
        conv.convert(new File(args[1]));
    }
}
